"""contains some default parameters of the application."""

FILES_PATH = '~/.pymultimatic'
"""Default file path for cookies and serial."""

COOKIE_FILE_NAME = '.cookies'
"""Default cookie file name."""

SERIAL_NUMBER_FILE_NAME = '.serial'
"""Default serial file name."""

SMARTPHONE_ID = 'pymultiMATIC'
"""Default smartphone id."""
